@@ -3,9 +3,9 @@
 ## author: Brice Ozenne
 ## created: feb 27 2017 (10:47) 
 ## Version: 
-## last-updated: Jan  6 2020 (08:59) 
-##           By: Thomas Alexander Gerds
-##     Update #: 84
+## last-updated: sep 24 2020 (14:30) 
+##           By: Brice Ozenne
+##     Update #: 103
 #----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -27,30 +27,41 @@
 #' @param reduce.data [logical] If \code{TRUE} only the covariates that does take indentical values for all observations are displayed.
 #' @param plot [logical] Should the graphic be plotted.
 #' @param digits [integer] Number of decimal places.
+#' @param smooth [logical] Should a smooth version of the risk function be plotted instead of a simple function?
 #' @param alpha [numeric, 0-1] Transparency of the confidence bands. Argument passed to \code{ggplot2::geom_ribbon}.
-#' @param ... Not used. Only for compatibility with the plot method.
+#' @param ... Additional parameters to cutomize the display.
+#' 
+#' @return Invisible. A list containing:
+#' \itemize{
+#' \item plot: the ggplot object.
+#' \item data: the data used to create the plot.
+#' }
+#'
+#' @seealso
+#' \code{\link{predict.CauseSpecificCox}} to compute risks based on a CSC model.
 
 ## * autoplot.predictCSC (examples)
 #' @examples
 #' library(survival)
 #' library(rms)
 #' library(ggplot2)
-##' library(prodlim)
+#' library(prodlim)
+#' 
 #' #### simulate data ####
 #' set.seed(10)
 #' d <- sampleData(1e2, outcome = "competing.risks")
-#'
+#' seqTau <- c(0,unique(sort(d[d$event==1,time])), max(d$time))
+#' 
 #' #### CSC model ####
 #' m.CSC <- CSC(Hist(time,event)~ X1 + X2 + X6, data = d)
 #' 
-#' pred.CSC <- predict(m.CSC, newdata = d[1:2,], time = 1:5, cause = 1)#'
-#' autoplot(pred.CSC)
+#' pred.CSC <- predict(m.CSC, newdata = d[1:2,], time = seqTau, cause = 1, band = TRUE)
+#' autoplot(pred.CSC, alpha = 0.2)
 #' 
-#'
 #' #### stratified CSC model ####
 #' m.SCSC <- CSC(Hist(time,event)~ strata(X1) + strata(X2) + X6,
 #'               data = d)
-#' pred.SCSC <- predict(m.SCSC, time = 1:3, newdata = d[1:4,],
+#' pred.SCSC <- predict(m.SCSC, time = seqTau, newdata = d[1:4,],
 #'                      cause = 1, keep.newdata = TRUE, keep.strata = TRUE)
 #' autoplot(pred.SCSC, group.by = "strata")
 
@@ -59,40 +70,46 @@
 #' @method autoplot predictCSC
 #' @export
 autoplot.predictCSC <- function(object,
-                                ci = FALSE,
-                                band = FALSE,
+                                ci = object$se,
+                                band = object$band,
+                                plot = TRUE,
+                                smooth = FALSE,
+                                digits = 2,
+                                alpha = NA,
                                 group.by = "row",
                                 reduce.data = FALSE,
-                                plot = TRUE,
-                                digits = 2, alpha = NA, ...){
+                                ...){
   
     ## initialize and check
     group.by <- match.arg(group.by, c("row","covariates","strata"))
   
     if(group.by[[1]] == "covariates" && ("newdata" %in% names(object) == FALSE)){
         stop("argument \'group.by\' cannot be \"covariates\" when newdata is missing in the object \n",
-             "set argment \'keep.newdata\' to TRUE when calling predictCox \n")
+             "set argment \'keep.newdata\' to TRUE when calling the predictCox function \n")
     }
     if(group.by[[1]] == "strata" && ("strata" %in% names(object) == FALSE)){
         stop("argument \'group.by\' cannot be \"strata\" when strata is missing in the object \n",
-             "set argment \'keep.strata\' to TRUE when calling predictCox \n")
+             "set argment \'keep.strata\' to TRUE when calling the predictCox function \n")
     }
   
     if(ci[[1]] && (object$se[[1]]==FALSE || is.null(object$conf.level))){
         stop("argument \'ci\' cannot be TRUE when no standard error have been computed \n",
-             "set arguments \'se\' and \'confint\' to TRUE when calling predict.CauseSpecificCox \n")
+             "set arguments \'se\' and \'confint\' to TRUE when calling the predict.CauseSpecificCox function \n")
     }
     if(band[[1]] && (object$band[[1]]==FALSE  || is.null(object$conf.level))){
         stop("argument \'band\' cannot be TRUE when the quantiles for the confidence bands have not been computed \n",
-             "set arguments \'band\' and \'confint\' to TRUE when calling predict.CauseSpecificCox \n")
+             "set arguments \'band\' and \'confint\' to TRUE when calling the predict.CauseSpecificCox function \n")
+    }
+    if(any(rank(object$times) != 1:length(object$times))){
+        stop("Invalid object. The prediction times must be strictly increasing \n")
     }
     
-    dots <- list(...)
-    if(length(dots)>0){
-        txt <- names(dots)
-        txt.s <- if(length(txt)>1){"s"}else{""}
-        stop("unknown argument",txt.s,": \"",paste0(txt,collapse="\" \""),"\" \n")
-    }
+    ## dots <- list(...)
+    ## if(length(dots)>0){
+    ##     txt <- names(dots)
+    ##     txt.s <- if(length(txt)>1){"s"}else{""}
+    ##     stop("unknown argument",txt.s,": \"",paste0(txt,collapse="\" \""),"\" \n")
+    ## }
     
 
     ## display
@@ -110,6 +127,7 @@ autoplot.predictCSC <- function(object,
                           outcome.lowerBand = if(band){object$absRisk.lowerBand}else{NULL},
                           outcome.upperBand = if(band){object$absRisk.upperBand}else{NULL},
                           newdata = newdata,
+                          status = NULL,
                           strata = object$strata,
                           times = object$times,
                           name.outcome = "absRisk",
@@ -124,7 +142,10 @@ autoplot.predictCSC <- function(object,
                            group.by = group.by,
                            conf.level = object$conf.level,
                            alpha = alpha,
-                           ylab = "absolute risk"
+                           smooth = smooth,
+                           xlab = "time",
+                           ylab = "absolute risk",
+                           ...
                            )
       
   if(plot){
